#!/bin/bash

###
# Startup script for VMware Fusion
# Copyright 2007, 2008 VMware, Inc.  All rights reserved.
###

###
# VM@Work Tokamak 2.0.0
# Modifications for mulitple interfaces
# Copyright Dave Parsons 2007 - 2008
# Updated for VMware Fusion 2.0 by Paul Rockwell
# Based on VMware materials & released with permission from VMware Inc.
###

set -e

# BEGINNING_OF_UTIL_DOT_SH
#!/bin/sh
#
# Copyright 2005-2008 VMware, Inc.  All rights reserved.
#
# A few utility functions used by our shell scripts.  Some expect the settings
# database to already be loaded and evaluated.

vmware_failed() {
  if [ "`type -t 'echo_failure' 2>/dev/null`" = 'function' ]; then
    echo_failure
  else
    echo -n "$rc_failed"
  fi
}

vmware_success() {
  if [ "`type -t 'echo_success' 2>/dev/null`" = 'function' ]; then
    echo_success
  else
    echo -n "$rc_done"
  fi
}

# Execute a macro
vmware_exec() {
  local msg="$1"  # IN
  local func="$2" # IN
  shift 2

  echo -n '   '"$msg"

  # On Caldera 2.2, SIGHUP is sent to all our children when this script exits
  # I wanted to use shopt -u huponexit instead but their bash version
  # 1.14.7(1) is too old
  #
  # Ksh does not recognize the SIG prefix in front of a signal name
  if [ "$VMWARE_DEBUG" = 'yes' ]; then
    (trap '' HUP; "$func" "$@")
  else
    (trap '' HUP; "$func" "$@") >/dev/null 2>&1
  fi
  if [ "$?" -gt 0 ]; then
    vmware_failed
    echo
    return 1
  fi

  vmware_success
  echo
  return 0
}

# Execute a macro in the background
vmware_bg_exec() {
  local msg="$1"  # IN
  local func="$2" # IN
  shift 2

  if [ "$VMWARE_DEBUG" = 'yes' ]; then
    # Force synchronism when debugging
    vmware_exec "$msg" "$func" "$@"
  else
    echo -n '   '"$msg"' (background)'

    # On Caldera 2.2, SIGHUP is sent to all our children when this script exits
    # I wanted to use shopt -u huponexit instead but their bash version
    # 1.14.7(1) is too old
    #
    # Ksh does not recognize the SIG prefix in front of a signal name
    (trap '' HUP; "$func" "$@") 2>&1 | logger -t 'VMware[init]' -p daemon.err &

    vmware_success
    echo
    return 0
  fi
}

# This is a function in case a future product name contains language-specific
# escape characters.
vmware_product_name() {
  echo '@@PRODUCT_NAME@@'
  exit 0
}

# This is a function in case a future product contains language-specific
# escape characters.
vmware_product() {
  echo '@@PRODUCT@@'
  exit 0
}

# They are a lot of small utility programs to create temporary files in a
# secure way, but none of them is standard. So I wrote this
make_tmp_dir() {
  local dirname="$1" # OUT
  local prefix="$2"  # IN
  local tmp
  local serial
  local loop

  tmp="${TMPDIR:-/tmp}"

  # Don't overwrite existing user data
  # -> Create a directory with a name that didn't exist before
  #
  # This may never succeed (if we are racing with a malicious process), but at
  # least it is secure
  serial=0
  loop='yes'
  while [ "$loop" = 'yes' ]; do
    # Check the validity of the temporary directory. We do this in the loop
    # because it can change over time
    if [ ! -d "$tmp" ]; then
      echo 'Error: "'"$tmp"'" is not a directory.'
      echo
      exit 1
    fi
    if [ ! -w "$tmp" -o ! -x "$tmp" ]; then
      echo 'Error: "'"$tmp"'" should be writable and executable.'
      echo
      exit 1
    fi

    # Be secure
    # -> Don't give write access to other users (so that they can not use this
    # directory to launch a symlink attack)
    if mkdir -m 0755 "$tmp"'/'"$prefix$serial" >/dev/null 2>&1; then
      loop='no'
    else
      serial=`expr $serial + 1`
      serial_mod=`expr $serial % 200`
      if [ "$serial_mod" = '0' ]; then
        echo 'Warning: The "'"$tmp"'" directory may be under attack.'
        echo
      fi
    fi
  done

  eval "$dirname"'="$tmp"'"'"'/'"'"'"$prefix$serial"'
}

# Removes "stale" device node
# On udev-based systems, this is never needed.
# On older systems, after an unclean shutdown, we might end up with
# a stale device node while the kernel driver has a new major/minor.
vmware_rm_stale_node() {
   local node="$1"  # IN
   if [ -e "/dev/$node" -a "$node" != "" ]; then
      local node_major=`ls -l "/dev/$node" | awk '{print \$5}' | sed -e s/,//`
      local node_minor=`ls -l "/dev/$node" | awk '{print \$6}'`
      if [ "$node_major" = "10" ]; then
         local real_minor=`cat /proc/misc | grep "$node" | awk '{print \$1}'`
         if [ "$node_minor" != "$real_minor" ]; then
            rm -f "/dev/$node"
         fi
      else
         local node_name=`echo $node | sed -e s/[0-9]*$//`
         local real_major=`cat /proc/devices | grep "$node_name" | awk '{print \$1}'`
         if [ "$node_major" != "$real_major" ]; then
            rm -f "/dev/$node"
         fi
      fi
   fi
}

# Checks if the given pid represents a live process.
# Returns 0 if the pid is a live process, 1 otherwise
vmware_is_process_alive() {
  local pid="$1" # IN

  ps -p $pid | grep $pid > /dev/null 2>&1
}

# Check if the process associated to a pidfile is running.
# Return 0 if the pidfile exists and the process is running, 1 otherwise
vmware_check_pidfile() {
  local pidfile="$1" # IN
  local pid

  pid=`cat "$pidfile" 2>/dev/null`
  if [ "$pid" = '' ]; then
    # The file probably does not exist or is empty. Failure
    return 1
  fi
  # Keep only the first number we find, because some Samba pid files are really
  # trashy: they end with NUL characters
  # There is no double quote around $pid on purpose
  set -- $pid
  pid="$1"

  vmware_is_process_alive $pid
}

# Note:
#  . Each daemon must be started from its own directory to avoid busy devices
#  . Each PID file doesn't need to be added to the installer database, because
#    it is going to be automatically removed when it becomes stale (after a
#    reboot). It must go directly under /var/run, or some distributions
#    (RedHat 6.0) won't clean it
#

# Terminate a process synchronously
vmware_synchrone_kill() {
   local pid="$1"    # IN
   local signal="$2" # IN
   local second

   kill -"$signal" "$pid"

   # Wait a bit to see if the dirty job has really been done
   for second in 0 1 2 3 4 5 6 7 8 9 10; do
      vmware_is_process_alive "$pid"
      if [ "$?" -ne 0 ]; then
         # Success
         return 0
      fi

      sleep 1
   done

   # Timeout
   return 1
}

# Kill the process associated to a pidfile
vmware_stop_pidfile() {
   local pidfile="$1" # IN
   local pid

   pid=`cat "$pidfile" 2>/dev/null`
   if [ "$pid" = '' ]; then
      # The file probably does not exist or is empty. Success
      return 0
   fi
   # Keep only the first number we find, because some Samba pid files are really
   # trashy: they end with NUL characters
   # There is no double quote around $pid on purpose
   set -- $pid
   pid="$1"

   # First try a nice SIGTERM
   if vmware_synchrone_kill "$pid" 15; then
      return 0
   fi

   # Then send a strong SIGKILL
   if vmware_synchrone_kill "$pid" 9; then
      return 0
   fi

   return 1
}

# Determine if SELinux is enabled
isSELinuxEnabled() {
   if [ "`cat /selinux/enforce 2> /dev/null`" = "1" ]; then
      echo "yes"
   else
      echo "no"
   fi
}

# Runs a command and retries under the provided SELinux context if it fails
vmware_exec_selinux() {
   local command="$1"
   # XXX We should probably ask the user at install time what context to use
   # when we retry commands.  unconfined_t is the correct choice for Red Hat.
   local context="unconfined_t"
   local retval

   $command
   retval=$?
   if [ $retval -ne 0 -a "`isSELinuxEnabled`" = 'yes' ]; then
      runcon -t $context -- $command
      retval=$?
   fi

   return $retval
}

# Start the blocking file system.  This consists of loading the module and
# mounting the file system.
vmware_start_vmblock() {
   mkdir -p /tmp/VMwareDnD && chmod 1777 /tmp/VMwareDnD
   vmware_exec 'Loading module' vmware_load_module $vmblock
   exitcode=`expr $exitcode + $?`
   vmware_exec_selinux "mount -t vmblock none /proc/fs/vmblock/mountPoint"
}

# Stop the blocking file system
vmware_stop_vmblock() {
   # It's okay for us to not factor the umount command's error code into
   # exitcode since the module unload will fail if the file system remains
   # mounted.
   vmware_exec_selinux "umount /proc/fs/vmblock/mountPoint"

   vmware_unload_module $vmblock
}

# This is necessary to allow udev time to create a device node.  If we don't
# wait then udev will override the permissions we choose when it creates the
# device node after us.
vmware_delay_for_node() {
   local node="$1"
   local delay="$2"

   while [ ! -e $node -a ${delay} -gt 0 ]; do 
      delay=`expr $delay - 1`
      sleep 1
   done
}

# starts after vmci is loaded
vmware_start_vsock() {
  if [ "`isLoaded "$vmci"`" = 'no' ]; then
    # vsock depends on vmci
    return 1
  fi
  vmware_load_module $vsock
  vmware_rm_stale_node vsock
  # Give udev 5 seconds to create our node
  vmware_delay_for_node "/dev/vsock" 5
  if [ ! -e /dev/vsock ]; then
     local minor=`cat /proc/misc | grep vsock | awk '{print $1}'`
     mknod --mode=666 /dev/vsock c 10 "$minor"
  else
     chmod 666 /dev/vsock
  fi

  return 0
}

# unloads before vmci
vmware_stop_vsock() {
  vmware_unload_module $vsock
  rm -f /dev/vsock
}

is_ESX_running() {
  if [ ! -f "$vmdb_answer_SBINDIR"/vmware-checkvm ] ; then
    echo no
    return
  fi
  if "$vmdb_answer_SBINDIR"/vmware-checkvm -p | grep -q ESX; then
    echo yes
  else
    echo no
  fi
}

#
# Start vmblock only if ESX is not running and the config script
# built/loaded it (kernel is >= 2.4.0 and  product is tools-for-linux).
#
is_vmblock_needed() {
  if [ "`is_ESX_running`" = 'yes' ]; then
    echo no
  else
    if [ "$vmdb_answer_VMBLOCK_CONFED" = 'yes' ]; then
      echo yes
    else
      echo no
    fi
  fi
}

wrap () {
  AMSG="$1"
  while [ `echo $AMSG | wc -c` -gt 75 ] ; do
    AMSG1=`echo $AMSG | sed -e 's/\(.\{1,75\} \).*/\1/' -e 's/  [ 	]*/  /'`
    AMSG=`echo $AMSG | sed -e 's/.\{1,75\} //' -e 's/  [ 	]*/  /'`
    echo "  $AMSG1"
  done
  echo "  $AMSG"
  echo " "
}

#---------------------------------------------------------------------------
#
# load_settings
#
# Load VMware Installer Service settings
#
# Returns:
#    0 on success, otherwise 1.
#
# Side Effects:
#    vmdb_* variables are set.
#---------------------------------------------------------------------------

load_settings() {
  local settings=`$DATABASE/vmis-settings`
  if [ $? -eq 0 ]; then
    eval "$settings"
    return 0
  else
    return 1
  fi
}

#---------------------------------------------------------------------------
#
# launch_binary
#
# Launch a binary with resolved dependencies.
#
# Returns:
#    None.
#
# Side Effects:
#    Process is replaced with the binary if successful,
#    otherwise returns 1.
#---------------------------------------------------------------------------

launch_binary() {
  local component="$1"		# IN: component name
  shift
  local binary="$2"		# IN: binary name
  shift
  local args="$@"		# IN: arguments
  shift

  # Convert -'s in component name to _ and lookup its libdir
  local component=`echo $component | tr '-' '_'`
  local libdir="vmdb_$component_libdir"

  exec "$libdir"'/bin/launcher.sh'		\
       "$libdir"'/lib'				\
       "$libdir"'/bin/'"$binary"		\
       "$libdir"'/libconf' "$args"
  return 1
}

# END_OF_UTIL_DOT_SH

# BEGINNING_OF_DB_DOT_SH
#!/bin/sh

#
# Manage an installer database
#

# Add an answer to a database in memory
db_answer_add() {
  local dbvar="$1" # IN/OUT
  local id="$2"    # IN
  local value="$3" # IN
  local answers
  local i

  eval "$dbvar"'_answer_'"$id"'="$value"'

  eval 'answers="$'"$dbvar"'_answers"'
  # There is no double quote around $answers on purpose
  for i in $answers; do
    if [ "$i" = "$id" ]; then
      return
    fi
  done
  answers="$answers"' '"$id"
  eval "$dbvar"'_answers="$answers"'
}

# Remove an answer from a database in memory
db_answer_remove() {
  local dbvar="$1" # IN/OUT
  local id="$2"    # IN
  local new_answers
  local answers
  local i

  eval 'unset '"$dbvar"'_answer_'"$id"

  new_answers=''
  eval 'answers="$'"$dbvar"'_answers"'
  # There is no double quote around $answers on purpose
  for i in $answers; do
    if [ "$i" != "$id" ]; then
      new_answers="$new_answers"' '"$i"
    fi
  done
  eval "$dbvar"'_answers="$new_answers"'
}

# Load all answers from a database on stdin to memory (<dbvar>_answer_*
# variables)
db_load_from_stdin() {
  local dbvar="$1" # OUT

  eval "$dbvar"'_answers=""'

  # read doesn't support -r on FreeBSD 3.x. For this reason, the following line
  # is patched to remove the -r in case of FreeBSD tools build. So don't make
  # changes to it.
  while read -r action p1 p2; do
    if [ "$action" = 'answer' ]; then
      db_answer_add "$dbvar" "$p1" "$p2"
    elif [ "$action" = 'remove_answer' ]; then
      db_answer_remove "$dbvar" "$p1"
    fi
  done
}

# Load all answers from a database on disk to memory (<dbvar>_answer_*
# variables)
db_load() {
  local dbvar="$1"  # OUT
  local dbfile="$2" # IN

  db_load_from_stdin "$dbvar" < "$dbfile"
}

# Iterate through all answers in a database in memory, calling <func> with
# id/value pairs and the remaining arguments to this function
db_iterate() {
  local dbvar="$1" # IN
  local func="$2"  # IN
  shift 2
  local answers
  local i
  local value

  eval 'answers="$'"$dbvar"'_answers"'
  # There is no double quote around $answers on purpose
  for i in $answers; do
    eval 'value="$'"$dbvar"'_answer_'"$i"'"'
    "$func" "$i" "$value" "$@"
  done
}

# If it exists in memory, remove an answer from a database (disk and memory)
db_remove_answer() {
  local dbvar="$1"  # IN/OUT
  local dbfile="$2" # IN
  local id="$3"     # IN
  local answers
  local i

  eval 'answers="$'"$dbvar"'_answers"'
  # There is no double quote around $answers on purpose
  for i in $answers; do
    if [ "$i" = "$id" ]; then
      echo 'remove_answer '"$id" >> "$dbfile"
      db_answer_remove "$dbvar" "$id"
      return
    fi
  done
}

# Add an answer to a database (disk and memory)
db_add_answer() {
  local dbvar="$1"  # IN/OUT
  local dbfile="$2" # IN
  local id="$3"     # IN
  local value="$4"  # IN

  db_remove_answer "$dbvar" "$dbfile" "$id"
  echo 'answer '"$id"' '"$value" >> "$dbfile"
  db_answer_add "$dbvar" "$id" "$value"
}

# Add a file to a database on disk
# 'file' is the file to put in the database (it may not exist on the disk)
# 'tsfile' is the file to get the timestamp from, '' if no timestamp
db_add_file() {
  local dbfile="$1" # IN
  local file="$2"   # IN
  local tsfile="$3" # IN
  local date

  if [ "$tsfile" = '' ]; then
    echo 'file '"$file" >> "$dbfile"
  else
    date=`date -r "$tsfile" '+%s' 2> /dev/null`
    if [ "$date" != '' ]; then
      date=' '"$date"
    fi
    echo 'file '"$file$date" >> "$dbfile"
  fi
}

# Remove file from database
db_remove_file() {
  local dbfile="$1" # IN
  local file="$2"   # IN

  echo "remove_file $file" >> "$dbfile"
}

# Add a directory to a database on disk
db_add_dir() {
  local dbfile="$1" # IN
  local dir="$2"    # IN

  echo 'directory '"$dir" >> "$dbfile"
}
# END_OF_DB_DOT_SH


# Start services on specified vnet
vmware_start_services() {
   local vHubNr="$1"  # IN

   local interface
   local hostaddr
   local netmask
   local dhcp
   local nat

   eval 'interface="$vmdb_answer_VNET_'"$vHubNr"'_INTERFACE"'
   eval 'hostaddr="$vmdb_answer_VNET_'"$vHubNr"'_HOSTONLY_HOSTADDR"'
   eval 'netmask="$vmdb_answer_VNET_'"$vHubNr"'_HOSTONLY_NETMASK"'
   eval 'dhcp="$vmdb_answer_VNET_'"$vHubNr"'_DHCP"'
   eval 'nat="$vmdb_answer_VNET_'"$vHubNr"'_NAT"'
   eval 'vmnet="vmnet'$vHubNr'"'

   if [ -n "$interface" ]; then
      # vmnet-bridge puts itself in the background (daemon mode)
      logger -s -t "VM@Work Tokamak 2.0.0" \
        "Starting: Program $LIBDIR/vmnet-bridge for $vmnet on interface $interface"
      "$LIBDIR/vmnet-bridge" -d "/var/run/vmnet-bridge-"$vmnet".pid" "$vmnet" "$interface"

   elif [ -n "$hostaddr" -a -n "$netmask" ]; then

      if [ "$nat" = 'yes' ]; then
        # vmnet-natd puts itself in the background (daemon mode)
        logger -s -t "VM@Work Tokamak 2.0.0" \
           "Starting: Program $LIBDIR/vmnet-natd for $vmnet using host address $hostaddr"

        # Make sure nat.conf is configured
        if [ ! -e "$LIBDIR/$vmnet/nat.conf" ]; then
           logger -s -t "VM@Work Tokamak 2.0.0" \
              "Error: $LIBDIR/$vmnet/nat.conf must exist!"
           exit 1
        fi

        "$LIBDIR/vmnet-natd" -c "$LIBDIR/"$vmnet"/nat.conf" \
           -m "$LIBDIR/"$vmnet"/nat.mac" \
           -d "/var/run/vmnet-natd-"$vmnet".pid" "$vmnet"
      fi
      
      if [ "$dhcp" = 'yes' -o "$nat" = 'yes' ]; then
        # vmnet-dhcpd puts itself in the background (daemon mode)
        logger -s -t "VM@Work Tokamak 2.0.0" \
           "Starting: Program $LIBDIR/vmnet-dhcpd for $vmnet using host address $hostaddr"

        # Make sure dhcpd.conf is configured
        if [ ! -e "$LIBDIR/$vmnet/dhcpd.conf" ]; then
           logger -s -t "VM@Work Tokamak 2.0.0" \
              "Error: $LIBDIR/$vmnet/dhcpd.conf must exist!"
           exit 1
        fi

        # Start the DHCP daemon
        "$LIBDIR/vmnet-dhcpd" -cf "$LIBDIR/$vmnet/dhcpd.conf" \
            -lf "$LEASEDIR"/vmnet-dhcpd-$vmnet.leases \
            -pf /var/run/vmnet-dhcpd-$vmnet.pid $vmnet
      fi

      # Create virtual interface for the virtual NIC
      logger -s -t "VM@Work Tokamak 2.0.0" \
         "Starting: Program $LIBDIR/vmnet-netifup for vmnet$vHubNr using host address $hostaddr"
      "$LIBDIR/vmnet-netifup" -d "/var/run/vmnet-netif-"$vmnet".pid" "$vmnet" "$vmnet"
      ifconfig $vmnet inet "$hostaddr" netmask "$netmask"

   fi
}

# Start services on specified vnet
vmware_stop_services() {
   local vHubNr="$1"  # IN

   local interface
   local hostaddr
   local netmask
   local dhcp
   local nat

   eval 'interface="$vmdb_answer_VNET_'"$vHubNr"'_INTERFACE"'
   eval 'hostaddr="$vmdb_answer_VNET_'"$vHubNr"'_HOSTONLY_HOSTADDR"'
   eval 'netmask="$vmdb_answer_VNET_'"$vHubNr"'_HOSTONLY_NETMASK"'
   eval 'dhcp="$vmdb_answer_VNET_'"$vHubNr"'_DHCP"'
   eval 'nat="$vmdb_answer_VNET_'"$vHubNr"'_NAT"'

   if [ -n "$interface" ]; then
      # Stop vmnet-bridge in the background (daemon mode)
      logger -s -t "VM@Work Tokamak 2.0.0" \
         "Stopping: Program $LIBDIR/vmnet-bridge for vmnet$vHubNr on interface $interface"
      vmware_stop_pidfile /var/run/vmnet-bridge-vmnet"$vHubNr".pid || true

   elif [ -n "$hostaddr" -a -n "$netmask" ]; then
   
      if [ "$dhcp" = 'yes' -o "$nat" = 'yes' ]; then      
         # Stop vmnet-dhcpd in the background (daemon mode)
        logger -s -t "VM@Work Tokamak 2.0.0" \
           "Stopping: Program $LIBDIR/vmnet-dhcpd for vmnet$vHubNr using host address $hostaddr"
        vmware_stop_pidfile /var/run/vmnet-dhcpd-vmnet"$vHubNr".pid || true
      fi

      # Stop virtual interface for the virtual NIC
      logger -s -t "VM@Work Tokamak 2.0.0" \
         "Stopping: Program $LIBDIR/vmnet-netifup for vmnet$vHubNr using host address $hostaddr"
      vmware_stop_pidfile /var/run/vmnet-netif-vmnet"$vHubNr".pid || true

      if [ "$nat" = 'yes' ]; then
        # Stop vmnet-natd in the background (daemon mode)
        logger -s -t "VM@Work Tokamak 2.0.0" \
           "Stopping: Program $LIBDIR/vmnet-natd for vmnet$vHubNr using host address $hostaddr"
        vmware_stop_pidfile /var/run/vmnet-natd-vmnet"$vHubNr".pid || true
      fi
   fi
}

# Print status of services on specified vnet
vmware_status_services() {
   local vHubNr="$1"  # IN

   local interface
   local hostaddr
   local netmask
   local dhcp
   local nat

   eval 'interface="$vmdb_answer_VNET_'"$vHubNr"'_INTERFACE"'
   eval 'hostaddr="$vmdb_answer_VNET_'"$vHubNr"'_HOSTONLY_HOSTADDR"'
   eval 'netmask="$vmdb_answer_VNET_'"$vHubNr"'_HOSTONLY_NETMASK"'
   eval 'dhcp="$vmdb_answer_VNET_'"$vHubNr"'_DHCP"'
   eval 'nat="$vmdb_answer_VNET_'"$vHubNr"'_NAT"'

   if [ -n "$interface" ]; then
      # vmnet-bridge puts itself in the background (daemon mode)
      echo -n 'Bridged networking on vmnet'"$vHubNr using $interface"
      if vmware_check_pidfile /var/run/vmnet-bridge-vmnet"$vHubNr".pid; then
	    echo ' is running'
      else
	    echo ' is not running'
      fi

   elif [ -n "$hostaddr" -a -n "$netmask" ]; then
        # Create virtual interface for the virtual NIC
	    echo -n 'Host-only/NAT networking on vmnet'"$vHubNr using $hostaddr/$netmask"
        if vmware_check_pidfile /var/run/vmnet-netif-vmnet"$vHubNr".pid; then
	      echo ' is running'
        else
	      echo ' is not running'
        fi

        if [ "$dhcp" = 'yes' -o "$nat" = 'yes' ]; then
        # vmnet-dhcpd puts itself in the background (daemon mode)
	    echo -n 'DHCP server on vmnet'"$vHubNr"
        if vmware_check_pidfile /var/run/vmnet-dhcpd-vmnet"$vHubNr".pid; then
	      echo ' is running'
        else
	      echo ' is not running'
        fi
      fi

      if [ "$nat" = 'yes' ]; then
        # vmnet-natd puts itself in the background (daemon mode)
	    echo -n 'NAT networking on vmnet'"$vHubNr"
        if vmware_check_pidfile /var/run/vmnet-natd-vmnet"$vHubNr".pid; then
	      echo ' is running'
        else
	      echo ' is not running'
        fi
      fi
   fi
}

export PATH=/bin:/sbin:/usr/bin:/usr/sbin

LIBDIR="/Library/Application Support/VMware Fusion"
LEASEDIR="/var/db/vmware"

# Write a logger message
logger -s -t "VM@Work Tokamak 2.0.0" "Extended network scripting - Dave Parsons"

#
# Since this script is installed, our main database should be installed too and
# should contain the basic information
vmware_db="$LIBDIR"/locations
if [ ! -r "$vmware_db" ]; then
   logger -s -t "VM@Work Tokamak 2.0.0" \
      "Warning: Unable to find main database $vmware_db."
   exit 1
fi

db_load 'vmdb' "$vmware_db"

# See how we were called.
case "$1" in
   --start)
   logger -s -t "VM@Work Tokamak 2.0.0" "Starting VMware Fusion: "

   if [ ! -x "$LIBDIR/vmnet-natd" -o ! -x "$LIBDIR/vmnet-dhcpd" ]; then
      logger -s -t "VM@Work Tokamak 2.0.0" \
         "Error: Programs $LIBDIR/vmnet-natd or $LIBDIR/vmnet-dhcpd not found!"
       exit 1
   fi
   if [ ! -x "$LIBDIR/vmnet-bridge" ]; then
      logger -s -t "VM@Work Tokamak 2.0.0" \
         "Error: Program $LIBDIR/vmnet-bridge not found!"
       exit 1
   fi
   if [ ! -x "$LIBDIR/vmnet-netifup" ]; then
      logger -s -t "VM@Work Tokamak 2.0.0" \
         "Error: Program $LIBDIR/vmnet-netifup not found!"
       exit 1
   fi


   kextload "$LIBDIR/kexts/vmmon.kext"
   kextload "$LIBDIR/kexts/vmci.kext"
   kextload "$LIBDIR/kexts/vmioplug.kext"
   kextload "$LIBDIR/kexts/vmnet.kext"

   mkdir -p "$LEASEDIR"
   chmod 755 "$LEASEDIR"

   # Loop thru possible vmnets
   # Constrained to 10 for now; i.e. 0-9
   # We have to do vmnet0 (bridged), vmnet1 (hostonly), & vmnet8 (nat)
   # first due to some internal Fusion dependnecies

   vmware_start_services "8"
   vmware_start_services "1"
   vmware_start_services "0"
   
   for ((i=2;i<=7;i+=1))
   do
      vmware_start_services "$i"
   done
   for ((i=9;i<=99;i+=1))
   do
      vmware_start_services "$i"
   done   
   
   # restart cisco vpn client
   # if we create interfaces inbetween the time the cisco vpn kext is
   # loaded and the time the cisco vpn client is started, the client
   # gets confused and terminates with error 51 (PR306839). If the
   # cisco vpn client is installed on this machine, we defensively
   # restart it.
   if [ -e "/System/Library/StartupItems/CiscoVPN/CiscoVPN" ]; then
      logger -s -t "VM@Work Tokamak 2.0.0" "restart cisco vpn client"
      /System/Library/StartupItems/CiscoVPN/CiscoVPN restart || true
   fi
   ;;
   

   --stop)
   logger -s -t "VM@Work Tokamak 2.0.0" "Shutting down VMware Fusion: "

   # Loop thru possible vmnets
   # Constrained to 10 for now; i.e. 0-9
   for ((i=0;i<=99;i+=1))
   do
      vmware_stop_services "$i"
   done

   # Remove the pid files
   rm -f /var/run/vmnet-*-vmnet*.pid

   # Now asynchronously kill any daemons started outside of this script.
   killall vmnet-natd || true
   killall vmnet-dhcpd || true
   killall vmnet-netifup || true
   killall vmnet-bridge || true

   # vmcrosstalk was renamed to vmci in Beta 3.
   kextunload -b com.vmware.kext.vmcrosstalk || true
   # vmmon's bundle was renamed to vmx86 in XXX.
   kextunload -b com.vmware.kext.vmmon || true

   kextunload "$LIBDIR/kexts/vmmon.kext" || true
   kextunload "$LIBDIR/kexts/vmci.kext" || true
   kextunload "$LIBDIR/kexts/vmioplug.kext" || true
   kextunload "$LIBDIR/kexts/vmnet.kext" || true
   echo
   ;;

   --restart)
   "$0" --stop && "$0" --start
   ;;

   --status)
   # Loop thru possible vmnets
   # Constrained to 10 for now; i.e. 0-9
   for ((i=0;i<=99;i+=1))
   do
      vmware_status_services "$i"
   done
   ;;
   *)
   echo "Usage: $0 {--start|--stop|--restart|--status}"
   exit 1
esac
